#############################################################
# 
# Print Zope source code as a big HTML file for easy reference
#
# 1. Put this file in Zope's Extensions directory
# 2. Create an external method in the root folder that calls 
#    the View function in this module. Give it an id like 
#    'inspect'
# 3. Try http://localhost:8080/inspect
#
# Warning: Browsing your hard drive via the web is a big
#   security risk. Use this only a local instance of
#	Zope for development purposes only!
#
# Author        Stephen Simmons, HealthArena B.V.
#               stephen.simmons@healtharena.net
#
# Copyright     22 September 2000
#
##############################################################

__doc__     = "Print Zope source code as a big HTML file for easy reference"
__version__ = "0.0.1"

file_order = [ 'README', '*.txt', '__init__.py', '*.py', '*.dtml', '*.c', '*.sh', '*.stx']

import sys, os, glob, fnmatch, fileinput, cgi, cStringIO

def ProcessDirectory(dir_name, dir_num=""):
    if str(dir_num)=="":
        print '<h2><a name="#L">'+ \
                os.path.basename(os.path.abspath(dir_name))+'</a></h2>'
        dir_link = ''
    else:
        dir_link = str(dir_num)+'.'
        
    dir_path = os.path.abspath(dir_name)
    glob_path = os.path.join(dir_path,"*")
    file_names = glob.glob(glob_path)
    file_list = []
    dir_list = []
    
    # Filter out directories to do after the real files
    for name in file_names[:]:
        if os.path.isdir( name):
            file_names.remove(name)
            dir_list.append(name)
    dir_list.sort()
    file_names.sort()
    
    # Reorder the file names by type
    for file_spec in file_order:
        for name in file_names[:]:
            if fnmatch.fnmatch(os.path.basename(name), file_spec):
                file_names.remove(name)
                file_list.append(name)

    # Print TOC entries
    file_num = 0
    for name in file_list+dir_list:
        file_num = file_num+1
        link_string = dir_link+str(file_num)
        print  '<a href="#L'+link_string+'">',
        print link_string+"&nbsp;&nbsp;"+os.path.basename(name)+'</a><br>'

    # Process each file in the directory
    file_num=0
    for name in file_list+dir_list:
        file_num=file_num+1
        link_string = dir_link+str(file_num)
        print '<h3><a name="#L'+link_string+'" href="#L'+str(dir_num)+'">'
        print link_string + '</a>'+'&nbsp;&nbsp;'+os.path.basename(name)+'</h3>'
        print "<p>("+name+")</p>"

        if name in dir_list:
            ProcessDirectory(name, link_string)
        else:
            # Print the contents of the file, cutting out the ZPL at the top of
            # each file if present. The ZPL and similar licences are detected with
            # - a line beginning with "##########" within the first 5 lines
            # - following lines begin with "#" on each successive line until
            # - a line beginning with "##########" after line 30
            print  "<pre>"
            cutting_ZPL = 0
            cut_lines = ''
            for line in fileinput.input(name):
                if not cutting_ZPL:
                    if line[:10]=='#' * 10 and fileinput.filelineno()<=5:
                        # Perhaps the start of a licence?
                        cut_note = "# (Cutting licence from line " + str(fileinput.filelineno())
                        cutting_ZPL = 1
                    else:
                        # Not in a licence, so print normally
                        print cgi.escape(line),
                if cutting_ZPL:
                    cut_lines = cut_lines + line
                    if line[0] != '#':
                        # No, this is not a licence, so print cut lines
                        print cgi.escape(cut_lines)
                        cut_lines = ''
                        cutting_ZPL = 0
                    elif line[:10]=='#' * 10 and fileinput.filelineno()>30:
                        # Yes, this is a licence, so print a note
                        print cut_note + " to line", str(fileinput.filelineno())+")"
                        cut_lines = ''
                        cutting_ZPL = 0
            if cut_lines <> '':
                print cgi.escape(cut_lines)
            print "</pre>"

def View(self, REQUEST=None):
    page = cStringIO.StringIO()
    sys.stdout = page
    print """
<html>
  <head><title>Zope source listing</title>
</head>
<body>
  <h2>Zope Source Listing</h2>
  <p>Version 0.0.1. 23 September 2000.</p>
  <p>Written by <a href=\"mailto:mail@stevesimmons.com\">Stephen Simmons</a></p>
"""
    if REQUEST.has_key('inspect') and REQUEST['inspect']!='':
        dir_path = os.path.abspath(os.path.join(REQUEST['inspect'],'..'))
        print '<p>Click <a href="?list=' + dir_path + '">here</a>'
        print 'to move back up to ' + dir_path + '.</p>'
        ProcessDirectory(REQUEST['inspect'])
    else:
        # Print list of directories
        if REQUEST.has_key('list') and REQUEST['list']!='':
            dir_path = os.path.abspath(REQUEST['list'])
        else:
            dir_path = os.path.abspath('.')
        glob_path = os.path.join(dir_path,"*")
        file_names = glob.glob(glob_path)
        parent_dir = os.path.abspath(os.path.join(dir_path,'..'))
        file_names.sort()
        print '<p>To display source code in ' + dir_path + ', click here:</p>'
        print '<ul><li><a href="?inspect=' + dir_path + '">' + dir_path + '</a></ul>'
        print '<p><b>Beware:</b> The source listing is recursive so starting high in the '
        print 'Zope tree can result in very large pages to display.</p>'
        dirs_shown = 0
        for name in file_names:
            if os.path.isdir(name):
                dirs_shown = dirs_shown + 1
                if dirs_shown==1:
                    print '<p>To look within a specific directory, '
                    print 'click on one of these:</p>\n<ul>'
                print '<li><a href="?list='+name+'">'+name+'</a><br>'
        if dirs_shown>0: print '</ul>'
        print '<p>To move up a directory, click here:</p>\n<ul>'
        print '<li><a href="?list='+parent_dir+'">'+parent_dir+'</a><br>'
        print '</ul>'
                
    print "</body></html>"
    return page.getvalue()
